package java.nio;

import sun.misc.Cleaner;
import sun.misc.Unsafe;
import sun.nio.ch.DirectBuffer;

class DirectShortBufferS extends ShortBuffer implements DirectBuffer
{
    protected static final Unsafe unsafe;
    private static final long arrayBaseOffset;
    protected static final boolean unaligned;
    private final Object att;
    
    @Override
    public Object attachment() {
        return this.att;
    }
    
    public Cleaner cleaner() {
        return null;
    }
    
    DirectShortBufferS(DirectBuffer att, int n, int n2, int n3, int n4, int n5) {
        super(n, n2, n3, n4);
        this.address = att.address() + (long)n5;
        this.att = att;
    }
    
    @Override
    public ShortBuffer slice() {
        int position = this.position();
        int limit = this.limit();
        int n = (position <= limit) ? (limit - position) : 0;
        int n2 = position << 1;
        assert n2 >= 0;
        return new DirectShortBufferS(this, -1, 0, n, n, n2);
    }
    
    @Override
    public ShortBuffer duplicate() {
        return new DirectShortBufferS(this, this.markValue(), this.position(), this.limit(), this.capacity(), 0);
    }
    
    @Override
    public ShortBuffer asReadOnlyBuffer() {
        return new DirectShortBufferRS(this, this.markValue(), this.position(), this.limit(), this.capacity(), 0);
    }
    
    @Override
    public long address() {
        return this.address;
    }
    
    private long ix(int n) {
        return this.address + ((long)n << 1);
    }
    
    @Override
    public short get() {
        return Bits.swap(DirectShortBufferS.unsafe.getShort(this.ix(this.nextGetIndex())));
    }
    
    @Override
    public short get(int n) {
        return Bits.swap(DirectShortBufferS.unsafe.getShort(this.ix(this.checkIndex(n))));
    }
    
    @Override
    public ShortBuffer get(short[] array, int n, int n2) {
        if ((long)n2 << 1 > 6L) {
            Buffer.checkBounds(n, n2, array.length);
            int position = this.position();
            int limit = this.limit();
            assert position <= limit;
            if (n2 > ((position <= limit) ? (limit - position) : 0)) {
                throw new BufferUnderflowException();
            }
            if (this.order() != ByteOrder.nativeOrder()) {
                Bits.copyToShortArray(this.ix(position), array, (long)n << 1, (long)n2 << 1);
            }
            else {
                Bits.copyToArray(this.ix(position), array, DirectShortBufferS.arrayBaseOffset, (long)n << 1, (long)n2 << 1);
            }
            this.position(position + n2);
        }
        else {
            super.get(array, n, n2);
        }
        return this;
    }
    
    @Override
    public ShortBuffer put(short n) {
        DirectShortBufferS.unsafe.putShort(this.ix(this.nextPutIndex()), Bits.swap(n));
        return this;
    }
    
    @Override
    public ShortBuffer put(int n, short n2) {
        DirectShortBufferS.unsafe.putShort(this.ix(this.checkIndex(n)), Bits.swap(n2));
        return this;
    }
    
    @Override
    public ShortBuffer put(ShortBuffer shortBuffer) {
        if ((shortBuffer instanceof DirectShortBufferS)) {
            if (shortBuffer == this) {
                throw new IllegalArgumentException();
            }
            DirectShortBufferS directShortBufferS = (DirectShortBufferS)shortBuffer;
            int position2 = directShortBufferS.position();
            int limit2 = directShortBufferS.limit();
            assert position2 <= limit2;
            int n2 = (position2 <= limit2) ? (limit2 - position2) : 0;
            int position3 = this.position();
            int limit3 = this.limit();
            assert position3 <= limit3;
            if (n2 > ((position3 <= limit3) ? (limit3 - position3) : 0)) {
                throw new BufferOverflowException();
            }
            DirectShortBufferS.unsafe.copyMemory(directShortBufferS.ix(position2), this.ix(position3), (long)n2 << 1);
            directShortBufferS.position(position2 + n2);
            this.position(position3 + n2);
        }
        else if (shortBuffer.hb != null) {
            int position = shortBuffer.position();
            int limit = shortBuffer.limit();
            assert position <= limit;
            int n = (position <= limit) ? (limit - position) : 0;
            this.put(shortBuffer.hb, shortBuffer.offset + position, n);
            shortBuffer.position(position + n);
        }
        else {
            super.put(shortBuffer);
        }
        return this;
    }
    
    @Override
    public ShortBuffer put(short[] array, int n, int n2) {
        if ((long)n2 << 1 > 6L) {
            Buffer.checkBounds(n, n2, array.length);
            int position = this.position();
            int limit = this.limit();
            assert position <= limit;
            if (n2 > ((position <= limit) ? (limit - position) : 0)) {
                throw new BufferOverflowException();
            }
            if (this.order() != ByteOrder.nativeOrder()) {
                Bits.copyFromShortArray(array, (long)n << 1, this.ix(position), (long)n2 << 1);
            }
            else {
                Bits.copyFromArray(array, DirectShortBufferS.arrayBaseOffset, (long)n << 1, this.ix(position), (long)n2 << 1);
            }
            this.position(position + n2);
        }
        else {
            super.put(array, n, n2);
        }
        return this;
    }
    
    @Override
    public ShortBuffer compact() {
        int position = this.position();
        int limit = this.limit();
        assert position <= limit;
        int n = (position <= limit) ? (limit - position) : 0;
        DirectShortBufferS.unsafe.copyMemory(this.ix(position), this.ix(0), (long)n << 1);
        this.position(n);
        this.limit(this.capacity());
        this.discardMark();
        return this;
    }
    
    @Override
    public boolean isDirect() {
        return true;
    }
    
    @Override
    public boolean isReadOnly() {
        return false;
    }
    
    @Override
    public ByteOrder order() {
        return (ByteOrder.nativeOrder() == ByteOrder.BIG_ENDIAN) ? ByteOrder.LITTLE_ENDIAN : ByteOrder.BIG_ENDIAN;
    }
    
    static {
        unsafe = Bits.unsafe();
        arrayBaseOffset = (long)DirectShortBufferS.unsafe.arrayBaseOffset(short[].class);
        unaligned = Bits.unaligned();
    }
}
